Package: statgenHTP
Title: High Throughput Phenotyping (HTP) Data Analysis
Version: 1.0.9.4
Date: 2026-07-06
Authors@R: 
    c(person(given = "Emilie J",
             family = "Millet",
             role = "aut",
             email = "emilie.millet@wur.nl",
             comment = c(ORCID = "0000-0002-2913-4892")),
      person(given = "Maria Xose",
             family = "Rodriguez Alvarez",
             role = "aut",
             comment = c(ORCID = "0000-0002-1329-9238")),
      person(given = "Diana Marcela",
             family = "Perez Valencia",
             role = "aut",
             comment = c(ORCID = "0000-0002-9053-2929")),
      person(given = "Isabelle",
             family = "Sanchez",
             role = "aut"),
      person(given = "Nadine",
             family = "Hilgert",
             role = "aut"),
      person(given = "Bart-Jan",
             family = "van Rossum",
             role = c("aut", "cre"),
             email = "bart-jan.vanrossum@wur.nl",
             comment = c(ORCID = "0000-0002-8673-2514")),
      person(given = "Fred", 
             family = "van Eeuwijk",
             role = "aut",
             comment = c(ORCID = "0000-0003-3672-2921")),             
      person(given = "Martin", 
             family = "Boer",
             role = "aut"))
Description: Phenotypic analysis of data coming from high throughput 
    phenotyping (HTP) platforms, including different types of outlier detection,
    spatial analysis, and parameter estimation. The package is being developed
    within the EPPN2020 project (<https://cordis.europa.eu/project/id/731013>).
    Some functions have been created to be used in conjunction with the R 
    package 'asreml' for the 'ASReml' software, which can be obtained upon 
    purchase from 'VSN' international (<https://vsni.co.uk/software/asreml-r/>).
License: GPL
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.6)
Imports: animation, ggforce, ggplot2 (>= 3.3.0), ggnewscale, grid,
        gridExtra, Matrix, methods, LMMsolver (>= 1.0.13), locfit,
        lubridate, rlang, scales, spam, SpATS (>= 1.0-13)
Suggests: asreml (>= 4.0), knitr, rmarkdown, tinytest (>= 1.2.4)
VignetteBuilder: knitr
URL: https://biometris.github.io/statgenHTP/index.html,
        https://github.com/Biometris/statgenHTP/
BugReports: https://github.com/Biometris/statgenHTP/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-06 10:22:22 UTC; rossu027
Author: Emilie J Millet [aut] (ORCID: <https://orcid.org/0000-0002-2913-4892>),
  Maria Xose Rodriguez Alvarez [aut] (ORCID:
    <https://orcid.org/0000-0002-1329-9238>),
  Diana Marcela Perez Valencia [aut] (ORCID:
    <https://orcid.org/0000-0002-9053-2929>),
  Isabelle Sanchez [aut],
  Nadine Hilgert [aut],
  Bart-Jan van Rossum [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8673-2514>),
  Fred van Eeuwijk [aut] (ORCID: <https://orcid.org/0000-0003-3672-2921>),
  Martin Boer [aut]
Maintainer: Bart-Jan van Rossum <bart-jan.vanrossum@wur.nl>
Repository: CRAN
Date/Publication: 2026-07-06 12:40:02 UTC
Built: R 4.5.2; ; 2026-07-06 14:15:41 UTC; unix
