# Generated by roxygen2: do not edit by hand

S3method("[",TP)
S3method("[",fitMod)
S3method(as.data.frame,TP)
S3method(plot,HTPSpline)
S3method(plot,TP)
S3method(plot,fitMod)
S3method(plot,psHDM)
S3method(plot,serieOut)
S3method(plot,singleOut)
S3method(plot,splineEst)
S3method(predict,psHDM)
S3method(summary,TP)
S3method(summary,fitMod)
export(countValid)
export(countValidPlot)
export(createTimePoints)
export(detectSerieOut)
export(detectSingleOut)
export(detectSingleOutMaize)
export(estimateSplineParameters)
export(fitModels)
export(fitSpline)
export(fitSplineHDM)
export(getCorrected)
export(getEffDims)
export(getGenoPred)
export(getHerit)
export(getTimePoints)
export(getVar)
export(removeSerieOut)
export(removeSingleOut)
export(removeTimePoints)
import(stats)
importFrom(LMMsolver,spl1D)
importFrom(SpATS,predict.SpATS)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,topo.colors)
importFrom(graphics,plot)
importFrom(methods,as)
importFrom(methods,new)
importFrom(rlang,.data)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,hasName)
importFrom(utils,packageVersion)
importFrom(utils,write.csv)
