
<!-- README.md is generated from README.Rmd. Please edit that file -->

# statgenHTP

[![](https://www.r-pkg.org/badges/version/statgenHTP)](https://www.r-pkg.org/pkg/statgenHTP)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/statgenHTP)](https://www.r-pkg.org/pkg/statgenHTP)
[![R-CMD-check](https://github.com/Biometris/statgenHTP/workflows/R-CMD-check/badge.svg)](https://github.com/Biometris/statgenHTP/actions?workflow=R-CMD-check)
[![codecov](https://codecov.io/gh/Biometris/statgenHTP/branch/main/graph/badge.svg)](https://app.codecov.io/gh/Biometris/statgenHTP)

**statgenHTP** is developed as an easy-to-use package for analyzing data
coming from high throughput phenotyping (HTP) platform experiments. The
package provides many options for plotting and exporting the results of
the analyses. It was developed within the [EPPN<sup>2020</sup>
project](https://cordis.europa.eu/project/id/731013) to meet the needs for
automated analyses of HTP data.

## Installation

- Install from CRAN:

``` r
install.packages("statgenHTP")
```

- Install latest development version from GitHub (requires
  [remotes](https://github.com/r-lib/remotes) package):

``` r
remotes::install_github("Biometris/statgenHTP", ref = "develop", dependencies = TRUE)
```
